(* Nova-Simulator
   ==============
   Frontpanel

    J. Rathlev, Schwentinental, (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   last modified:  August 2017
   *)

unit FpNova3D;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, Switches, Indicators;

const
  BusWidth = 16;

type
  TNovaCommands = (ncDepAc0,ncExAc0,ncDepAc1,ncExAc1,ncDepAc2,ncExAc2,ncDepAc3,ncExAc3,
                   ncReset,ncStop,ncStart,ncCont,ncDep,ncDepNext,ncEx,ncExNext,
                   ncMemStep,ncInstStep,ncProgLoad);

  TGetWordProc = procedure(Value : word) of object;
  TGetCommandProc = procedure(Sender : TObject; Command : TNovaCommands) of object;

  TNovaFrontpanel = class(TForm)
    UpperFrontPanel: TPanel;
    LowerFrontPanel: TPanel;
    UfpImage: TImage;
    AddressLabel: TLabel;
    DataLabel: TLabel;
    CarryLabel: TLabel;
    RunLabel: TLabel;
    IowLabel: TLabel;
    FetchLabel: TLabel;
    DeferLabel: TLabel;
    ExeLabel: TLabel;
    PowerLabel: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    RunLamp: TLamp3D;
    IOWLamp: TLamp3D;
    FetchLamp: TLamp3D;
    DeferLamp: TLamp3D;
    ExecuteLamp: TLamp3D;
    Ac0Switch: TBiMomSwitch3D;
    Ac1Switch: TBiMomSwitch3D;
    Ac2Switch: TBiMomSwitch3D;
    Ac3Switch: TBiMomSwitch3D;
    ResetSwitch: TBiMomSwitch3D;
    StartSwitch: TBiMomSwitch3D;
    DepositSwitch: TBiMomSwitch3D;
    ExamineSwitch: TBiMomSwitch3D;
    StepSwitch: TBiMomSwitch3D;
    ProgSwitch: TBiMomSwitch3D;
    Label18: TLabel;
    LfpImage: TImage;
    BasePanel: TPanel;
    Label1: TLabel;
    Label19: TLabel;
    BpImage: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Ac0SwitchDownClick(Sender: TObject);
    procedure Ac0SwitchUpClick(Sender: TObject);
    procedure Ac1SwitchDownClick(Sender: TObject);
    procedure Ac1SwitchUpClick(Sender: TObject);
    procedure Ac2SwitchDownClick(Sender: TObject);
    procedure Ac2SwitchUpClick(Sender: TObject);
    procedure Ac3SwitchDownClick(Sender: TObject);
    procedure Ac3SwitchUpClick(Sender: TObject);
    procedure ResetSwitchDownClick(Sender: TObject);
    procedure ResetSwitchUpClick(Sender: TObject);
    procedure StartSwitchDownClick(Sender: TObject);
    procedure StartSwitchUpClick(Sender: TObject);
    procedure DepositSwitchDownClick(Sender: TObject);
    procedure DepositSwitchUpClick(Sender: TObject);
    procedure ExamineSwitchDownClick(Sender: TObject);
    procedure ExamineSwitchUpClick(Sender: TObject);
    procedure StepSwitchDownClick(Sender: TObject);
    procedure StepSwitchUpClick(Sender: TObject);
    procedure ProgSwitchUpClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure UfpImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    AddressLamps : array [0..BusWidth-2] of TLamp3D;
    DataLamps    : array [0..BusWidth-1] of TLamp3D;
    DataSwitches : array [0..BusWidth-1] of TSwitch3D;
    DataLabels   :  array [0..BusWidth-1] of TLabel;
    CarryLamp    : TLamp3D;
    FOnChangeDataSwitch : TGetWordProc;
    FOnChangeCommandSwitch: TGetCommandProc;
    FAddress,
    FSwitches : word;
    FIniName,FIniSection : string;
    procedure ChangeSwitch (Sender : TObject);
    function GetRun : boolean;
    procedure SetRun (Value : boolean);
    procedure SetAddress (Value : word);
    procedure SetCarry (Value : boolean);
    procedure SetData (Value : word);
    procedure SetSwitches (Value : word);
    function GetSwitches : word;
  public
    { Public declarations }
    procedure LoadFromIni(IniName, Section : string);
    property Run : boolean read GetRun write SetRun;
    property Address : word read FAddress write SetAddress;
    property Carry : boolean write SetCarry;
    property Data: word write SetData;
    property Switches : word read GetSwitches write SetSwitches;
    property OnChangeDataSwitch : TGetWordProc read FOnChangeDataSwitch write FOnChangeDataSwitch;
    property OnChangeCommandSwitch : TGetCommandProc read FOnChangeCommandSwitch write FOnChangeCommandSwitch;
  end;

var
  NovaFrontpanel: TNovaFrontpanel;

implementation

{$R *.DFM}

uses IniFiles, GnuGetText, NovaMain;

const
  USize = 16;
  UTop =35;
  UVDelta1 = 40;
  URDelta1 = UVDelta1 div 2 -2;
  UVDelta2 = 60;
  URDelta2 = UVDelta2-URDelta1-4;
  ULeft = 215;
  UHDelta1 = 30;
  UHDelta2 = 20;
  UHDelta3 = 85;

procedure TNovaFrontpanel.FormCreate(Sender: TObject);
var
  i,x1,x2,x,y,d : integer;
begin
  TranslateComponent(self);
  with BpImage,Canvas do begin     // DG-Logo
    Brush.Color:=BasePanel.Color;
    FillRect(Rect(0,0,Width-1,Height-1));
    with Pen do begin
      Color:=clCream; Width:=2;
      end;
    RoundRect(2,2,Width-3,Height-3,5,5);
    x:=50; y:=Height div 2; d:=8;
    Chord(x-d,y-d,x+d,y+d,x,y-d,x,y+d);
    Brush.Color:=clCream;
    x:=54;
    Pie(x-d,y-d,x+d,y+d,x,y+d,x+d,y);
    Brush.Color:=BasePanel.Color;
    end;
  with UfpImage,Canvas do begin
    Brush.Color:=UpperFrontPanel.Color;
    FillRect(Rect(0,0,Width-1,Height-1));
    with Pen do begin
      Color:=clRed; Width:=2;
      end;
    RoundRect(2,2,Width-3,Height-3,5,5);
    with Pen do begin
      Color:=$00949494; Width:=2;
      end;
    Brush.Color:=$00949494;
    x:=60; y:=90; d:=20;
    Ellipse(x-d,y-d,x+d,y+d);
    Pen.Color:=clSilver;
    Brush.Color:=clSilver;
    d:=16;
    Ellipse(x-d,y-d,x+d,y+d);
    Pen.Color:=clBlack;
    d:=10;
    Ellipse(x-d,y-d,x+d,y+d);
    MoveTo(x,y-d div 2); LineTo(x,y-d);
    with Pen do begin
      Color:=clRed; Width:=2;
      end;
    Brush.Color:=UpperFrontPanel.Color;
    y:=UTop-Top+1;
    x1:=ULeft-UHDelta3-Left+1;
    x2:=ULeft-Left+1+UHDelta1*BusWidth+UHDelta2*((BusWidth+1) div 3);
    RoundRect(x1,y-URDelta1,x2,y+URDelta1,5,5);
    for i:=0 to (BusWidth+1) div 3 -1 do begin
      x:=ULeft-Left+1+UHDelta1+(3*UHDelta1+UHDelta2)*i;
      MoveTo(x,y-URDelta1); LineTo(x,y+URDelta1-1);
      end;
    y:=UTop+UVDelta1-Top+1;
    RoundRect(x1,y-URDelta1,x2,y+URDelta1,5,5);
    for i:=0 to (BusWidth+1) div 3 -1 do begin
      x:=ULeft-Left+1+UHDelta1+(3*UHDelta1+UHDelta2)*i;
      MoveTo(x,y-URDelta1); LineTo(x,y+URDelta1-1);
      end;
    y:=UTop+UVDelta1+UVDelta2-Top+1;
    RoundRect(x1,y-URDelta2,x2,y+URDelta1,5,5);
    for i:=0 to (BusWidth+1) div 3 -1 do begin
      x:=ULeft-Left+1+UHDelta1+(3*UHDelta1+UHDelta2)*i;
      MoveTo(x,y-URDelta2); LineTo(x,y+URDelta1-1);
      end;
    end;
  for i:=0 to BusWidth-2 do begin
    AddressLamps[i]:=TLamp3D.Create(self);
    with AddressLamps[i] do begin
      Parent:=UpperFrontPanel;
      GlyphSize:=gs16;
      Top:=UTop-USize div 2;
      Left:=ULeft+UHDelta2+UHDelta1*(i+1)+UHDelta2*(i div 3); Width:=USize;
      LightOn:=i mod 2 =0; //true;
      end;
    end;
  with AddressLabel do begin
    Left:=ULeft-UHDelta3+7;
    Top:=UTop-abs(Font.Height) div 2;
    end;
  for i:=0 to BusWidth-1 do begin
    DataLamps[i]:=TLamp3D.Create(self);
    with DataLamps[i] do begin
      Parent:=UpperFrontPanel;
      GlyphSize:=gs16;
      Top:=UTop+UVDelta1-USize div 2; Height:=USize;
      Left:=ULeft+UHDelta1*i+UHDelta2*((i+2) div 3); Width:=USize;
      LightOn:=true;
      end;
    end;
  with DataLabel do begin
    Left:=ULeft-UHDelta3+7;
    Top:=UTop+UVDelta1-abs(Font.Height) div 2;
    end;
  y:=UTop+UVDelta1+UVDelta2-USize div 2-10;
  for i:=0 to BusWidth-1 do begin
    x:=ULeft+UHDelta1*i+UHDelta2*((i+2) div 3);
    DataSwitches[i]:=TSwitch3D.Create(self);
    with DataSwitches[i] do begin
      Parent:=UpperFrontPanel;
      GlyphHeight:=gh32;
      Top:=y; //Height:=USize;
      Left:=x; //Width:=USize;
      OnDownClick:=ChangeSwitch;
      OnUpClick:=ChangeSwitch;
      end;
    DataLabels[i]:=TLabel.Create(self);
    with DataLabels[i] do begin
      Parent:=UpperFrontPanel;
      with Font do begin
        Color:=clWhite;
        Size:=6;
        end;
      Caption:=IntToStr(i);
      Width:=20; Height:=10;
      Left:=x+USize div 2-Width div 2;
      Top:=y-URDelta2 div 2+10;
      AutoSize:=false;
      Alignment:=taCenter;
      end;
    end;
  with CarryLabel do begin
    Left:=ULeft-2*UHDelta1+USize div 2;
    Top:=y-URDelta2 div 2+10;
    end;
  CarryLamp:=TLamp3D.Create(self);
  with CarryLamp do begin
    Parent:=UpperFrontPanel;
    GlyphSize:=gs16;
    Top:=UTop+UVDelta1-USize div 2;; Height:=USize;
    Left:=ULeft-UHDelta1; Width:=USize;
    LightOn:=true;
    end;
  with LfpImage.Canvas do begin
    Brush.Color:=LowerFrontPanel.Color;
    FillRect(Rect(0,0,Width-1,Height-1));
    with Pen do begin
      Color:=clBlack;
      Width:=1;
      end;
    Brush.Color:=clBlack;
    with Ac0Switch do begin
      x:=Left-20;
      y:=Top+Height div 2-4;
      MoveTo(x,y-15); LineTo(x,y+15);
      Ellipse(x-3,y-17,x+4,y-10);
      MoveTo(x,y); LineTo(Left-2,y);
      Ellipse(x-3,y+10,x+4,y+17);
      MoveTo(Left+Width-8,y);
      end;
    with Ac1Switch do begin
      LineTo(Left-2,y);
      MoveTo(Left+Width-8,y);
      end;
    with Ac2Switch do begin
      LineTo(Left-2,y);
      MoveTo(Left+Width-8,y);
      end;
    with Ac3Switch do begin
      LineTo(Left-2,y);
      x:=Left+Width+12;
      MoveTo(Left+Width-8,y); LineTo(x,y);
      MoveTo(x,y-15); LineTo(x,y+15);
      Ellipse(x-3,y-17,x+4,y-10);
      MoveTo(x,y); LineTo(Left-2,y);
      Ellipse(x-3,y+10,x+4,y+17);
      end;
    end;
//  LowerFrontPanel.ParentBackground:=false;
  FIniName:=''; FIniSection:='';
  Switches:=0; FAddress:=0;
  end;

const
  iniTop = 'Top';
  iniLeft = 'Left';

procedure TNovaFrontpanel.LoadFromIni(IniName,Section : string);
begin
  if (length(IniName)>0) and (length(Section)>0) then begin
    FIniName:=IniName; FIniSection:=Section;
    with TIniFile.Create(FIniName) do begin
      Top:=ReadInteger(FIniSection,iniTop,Top);
      Left:=ReadInteger(FIniSection,iniLeft,Left);
      Free;
      end;
    end;
  end;

procedure TNovaFrontpanel.FormDestroy(Sender: TObject);
var
  i : integer;
begin
  if (length(FIniName)>0) and (length(FIniSection)>0) then begin
    with TIniFile.Create(FIniName) do begin
      WriteInteger(FIniSection,iniTop,Top);
      WriteInteger(FIniSection,iniLeft,Left);
      Free;
      end;
    end;
  for i:=0 to BusWidth-2 do AddressLamps[i].Free;
  for i:=0 to BusWidth-1 do DataLamps[i].Free;
  for i:=0 to BusWidth-1 do DataSwitches[i].Free;
  end;

function TNovaFrontpanel.GetRun : boolean;
begin
  Result:=RunLamp.LightOn;
  end;

procedure TNovaFrontpanel.SetRun (Value : boolean);
begin
  RunLamp.LightOn:=Value;
  end;

procedure TNovaFrontpanel.SetAddress (Value : word);
var
  i : integer;
begin
  FAddress:=Value;
  for i:=BusWidth-2 downto 0 do with AddressLamps[i] do begin
    LightOn:=Value and 1 <>0;
    Value:=Value shr 1;
    end;
  end;

procedure TNovaFrontpanel.SetCarry (Value : boolean);
begin
  CarryLamp.LightOn:=Value;
  end;

procedure TNovaFrontpanel.SetData (Value : word);
var
  i : integer;
begin
  for i:=BusWidth-1 downto 0 do with DataLamps[i] do begin
    LightOn:=Value and 1 <>0;
    Value:=Value shr 1;
    end;
  end;

procedure TNovaFrontpanel.SetSwitches (Value : word);
var
  i : integer;
begin
  if Value<>FSwitches then begin
    FSwitches:=Value;
    for i:=BusWidth-1 downto 0 do with DataSwitches[i] do begin
      if Value and 1 <>0 then State:=ssOn else State:=ssOff;
      Value:=Value shr 1;
      end;
    end;
  end;

function TNovaFrontpanel.GetSwitches : word;
var
  i : integer;
begin
  Result:=0;
  for i:=0 to BusWidth-1 do with DataSwitches[i] do begin
    Result:=Result shl 1;
    if State=ssOn then Result:=Result or 1;
    end;
  FSwitches:=Result;
  end;

procedure TNovaFrontpanel.ChangeSwitch (Sender : TObject);
begin
  if assigned(FOnChangeDataSwitch) then FOnChangeDataSwitch(GetSwitches);
  end;

procedure TNovaFrontpanel.Ac0SwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncExAc0);
  end;

procedure TNovaFrontpanel.Ac0SwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncDepAc0);
  end;

procedure TNovaFrontpanel.Ac1SwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncExAc1);
  end;

procedure TNovaFrontpanel.Ac1SwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncDepAc1);
  end;

procedure TNovaFrontpanel.Ac2SwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncExAc2);
  end;

procedure TNovaFrontpanel.Ac2SwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncDepAc2);
  end;

procedure TNovaFrontpanel.Ac3SwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncExAc3);
  end;

procedure TNovaFrontpanel.Ac3SwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncDepAc3);
  end;

procedure TNovaFrontpanel.ResetSwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncStop);
  end;

procedure TNovaFrontpanel.ResetSwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncReset);
  end;

procedure TNovaFrontpanel.StartSwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncCont);
  end;

procedure TNovaFrontpanel.StartSwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncStart);
  end;

procedure TNovaFrontpanel.DepositSwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncDepNext);
  end;

procedure TNovaFrontpanel.DepositSwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncDep);
  end;

procedure TNovaFrontpanel.ExamineSwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncExNext);
  end;

procedure TNovaFrontpanel.ExamineSwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncEx);
  end;

procedure TNovaFrontpanel.StepSwitchDownClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncMemStep);
  end;

procedure TNovaFrontpanel.StepSwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncInstStep);
  end;

function IsInCircle (Pt,Center : TPoint; Radius : integer) : boolean;
begin
  Result:=sqrt(sqr(Pt.X-Center.X)+sqr(Pt.Y-Center.Y))<Radius;
  end;

function IsInRect (Pt : TPoint; R : TRect) : boolean;
begin
  with R,Pt do Result:=(x>=Left) and (x<=Right) and (y>=Top) and (y<=Bottom);
  end;

procedure TNovaFrontpanel.UfpImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if IsInCircle(Point(x,y),Point(60,90),20) then MainForm.btQuit.Click;
  end;

procedure TNovaFrontpanel.ProgSwitchUpClick(Sender: TObject);
begin
  if assigned(FOnChangeCommandSwitch) then FOnChangeCommandSwitch(self,ncProgLoad);
  end;

procedure TNovaFrontpanel.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  MainForm.itmFrontPanel.Checked:=false;
  end;

end.
