(* Nova-Simulator
   ==============
   Value input

    J. Rathlev, Schwentinental, (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   last modified:  August 2017
   *)

unit InpValue;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, NumberUtils, NumberEd;

type
  TValueDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    lbDesc: TLabel;
    edValue: TNumberEdit;
    GroupBox3: TGroupBox;
    rbDecimal: TRadioButton;
    rbOctal: TRadioButton;
    rbHex: TRadioButton;
    rbBin: TRadioButton;
    cbBytes: TCheckBox;
    procedure cbBytesClick(Sender: TObject);
    procedure rbBinClick(Sender: TObject);
    procedure rbOctalClick(Sender: TObject);
    procedure rbDecimalClick(Sender: TObject);
    procedure rbHexClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function Execute (Pos : TPoint; Desc : string; ANumMode : TNumMode;
                      var AValue : word) : boolean;
  end;

var
  ValueDialog: TValueDialog;

implementation

{$R *.DFM}

uses WinUtils, GnuGetText;

procedure TValueDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent(self);
  end;

procedure TValueDialog.cbBytesClick(Sender: TObject);
begin
  edValue.Bytes:=cbBytes.Checked;
  if Active then edValue.SetFocus;
  end;

procedure TValueDialog.rbBinClick(Sender: TObject);
begin
  edValue.NumMode:=nmBin;
  if Active then edValue.SetFocus;
  end;

procedure TValueDialog.rbOctalClick(Sender: TObject);
begin
  edValue.NumMode:=nmOctal;
  if Active then edValue.SetFocus;
  end;

procedure TValueDialog.rbDecimalClick(Sender: TObject);
begin
  edValue.NumMode:=nmDecimal;
  if Active then edValue.SetFocus;
  end;

procedure TValueDialog.rbHexClick(Sender: TObject);
begin
  edValue.NumMode:=nmHex;
  if Active then edValue.SetFocus;
  end;

function TValueDialog.Execute (Pos : TPoint; Desc : string; ANumMode : TNumMode;
                               var AValue : word) : boolean;
begin
  with Pos do if X<0 then Position:=poDesktopCenter
  else begin
    Position:=poDesigned;
    CheckScreenBounds(Screen,x,y,Width,Height);
    Left:=x; Top:=y;
    end;
  lbDesc.Caption:=Desc;
  with edValue do begin
    NumMode:=ANumMode;
    Value:=AValue;
    end;
  case ANumMode of
  nmBin : rbBin.Checked:=true;
  nmOctal : rbOctal.Checked:=true;
  nmHex : rbHex.Checked:=true;
  else rbDecimal.Checked:=true;
    end;
  cbBytes.Checked:=false;
  ActiveControl:=edValue;
  edValue.SelectAll;
  if ShowModal=mrOK then begin
    AValue:=edValue.Value;
    Result:=true;
    end
  else Result:=false;
  end;

end.
