(* Nova-Hilfsprogramm
   ==================
   1. Binary check
   2. ASCII convert (remove parity, $7F)

    J. Rathlev, Schwentinental, (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 -  Mar. 2010
   last modified:  May 2024
   *)

unit NovaCheckMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.StdCtrls,
  Vcl.Buttons;

const
  ProgName = 'Check NOVA files';
  Vers = '2.0';
  CopRgt = ' 2007-2024 Dr. J. Rathlev';
  EmailAdr = 'kontakt(a)rathlev-home.de';

  BinExt = 'bin';
  AsmExt = 'asm';
  TxtExt = 'txt';
  TmpExt = '.tmp';
  BakExt = '.bak';

type
  THauptForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    SourceSelectBtn: TSpeedButton;
    FileBtn: TSpeedButton;
    rgMode: TRadioGroup;
    InfoBtn: TSpeedButton;
    btStart: TSpeedButton;
    EndeBtn: TBitBtn;
    chbSubDir: TCheckBox;
    lbResult: TListBox;
    Panel1: TPanel;
    OpenDialog: TOpenDialog;
    cbShow: TCheckBox;
    FileEdit: TComboBox;
    SourceDirEdit: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure rgModeClick(Sender: TObject);
    procedure SourceSelectBtnClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure btStartClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    AppPath,UserPath,
    IniName          : string;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.dfm}

uses Winapi.ShlObj, System.IniFiles, ShellDirDlg, WinUtils, MsgDialogs, StringUtils,
  WinApiUtils, NumberUtils, PathUtils, GnuGetText, LangUtils, InitProg, NovaUtils;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  DirSekt = 'Directories';
  FileSekt = 'Files';

  (* INI-Variablen *)
  iniMode = 'Mode';
  iniSubDir = 'Subdirectories';

procedure THauptForm.FormCreate(Sender: TObject);
begin
  TranslateComponent (self);
  InitPaths (AppPath,UserPath);
  InitVersion (ProgName,Vers,CopRgt,3,3,ProgVersName,ProgVersDate);
  Caption:=_('Check NOVA files')+' - '+VersInfo.Comments;
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  with TMemIniFile.Create(IniName) do begin
    rgMode.ItemIndex:=ReadInteger(CfgSekt,iniMode,0);
    chbSubDir.Checked:=ReadBool(CfgSekt,iniSubDir,false);
    Free;
    end;
  LoadHistory(IniName,DirSekt,SourceDirEdit);
  LoadHistory(IniName,FileSekt,FileEdit);
  if FileEdit.Items.Count=0 then AddToHistory(FileEdit,'*.'+BinExt);
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
begin
  with TMemIniFile.Create(IniName) do begin
    WriteInteger(CfgSekt,iniMode,rgMode.ItemIndex);
    WriteBool(CfgSekt,iniSubDir,chbSubDir.Checked);
    UpdateFile;
    Free;
    end;
  SaveHistory(IniName,DirSekt,true,SourceDirEdit);
  SaveHistory(IniName,FileSekt,true,FileEdit);
  end;

procedure THauptForm.FormShow(Sender: TObject);
begin
  rgModeClick(Sender);
  end;

procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog (ProgName,Caption+ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr);
  end;

procedure THauptForm.rgModeClick(Sender: TObject);
begin
  if rgMode.ItemIndex=0 then FileEdit.Text:='*.'+BinExt
  else FileEdit.Text:='*.'+TxtExt+';*.'+AsmExt;
  end;

procedure THauptForm.SourceSelectBtnClick(Sender: TObject);
var
  s,t : string;
begin
  s:=SourceDirEdit.Text;
  if length(s)=0 then s:=UserPath;
  if rgMode.ItemIndex=0 then t:=_('Directory holding binary files')
  else t:=_('Directory holding text files');
  if ShellDirDialog.Execute (t,false,true,false,UserPath,s) then begin
    AddToHistory(SourceDirEdit,s);
    rgModeClick(Sender);
    end;
  end;

procedure THauptForm.FileBtnClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  with OpenDialog do begin
    if rgMode.ItemIndex=0 then begin
      Filter:=Format(_('Binary files|*.%s|all|*.*'),[BinExt]);
      Title:=_('Select binary files');
      end
    else begin
      Filter:=Format(_('Text files|*.%s|Assembler sources|*.%s|all|*.*'),[TxtExt,AsmExt]);
      Title:=_('Select text files');
      end;
    Filename:='';
    InitialDir:=IncludeTrailingPathDelimiter(SourceDirEdit.Text);
    if Execute then begin
      chbSubDir.Checked:=false;
      s:=ExtractFilePath(Files.Strings[0]);
      AddToHistory(SourceDirEdit,s);
      with Files do begin
        s:='';
        for i:=0 to Count-1 do s:=s+ExtractFilename(Strings[i])+';';
        end;
      delete (s,length(s),1);
      AddToHistory(FileEdit,s);
      end;
    end;
  end;

procedure THauptForm.btStartClick(Sender: TObject);
var
  FListe          : TStringList;
  fcnt,i,j,m,n,k  : integer;
  s,t,sd,
  fname,oname     : string;
  InStream,
  OutStream       : TFileStream;
  w,wa,cs         : word;
  b               : byte;
  sp              : int64;
  ok              : boolean;

  procedure ScanDir (Dir,Mask : string);
  var
    DirInfo    : TSearchRec;
    Findresult : integer;
  begin
    if chbSubDir.Checked then begin
      FindResult:=FindFirst (Erweiter(Dir,'*','*'),faAnyFile,DirInfo);
      while FindResult=0 do with DirInfo do begin
        if NotSpecialDir(Name) then begin
          if((Attr and faDirectory)<>0) then ScanDir (Erweiter(Dir,Name,''),Mask);
          end;
        FindResult:=FindNext (DirInfo);
        end;
      FindClose(DirInfo);
      end;
    FindResult:=FindFirst (Erweiter(Dir,Mask,''),faArchive+faReadOnly+faHidden+faSysfile,DirInfo);
    while FindResult=0 do with DirInfo do begin
      if NotSpecialDir(Name) then FListe.Add(Erweiter(Dir,Name,''));
      FindResult:=FindNext (DirInfo);
      end;
    FindClose(DirInfo);
    end;

begin
  t:=FileEdit.Text;
  if length(t)>0 then begin
    Screen.Cursor:=crHourglass;
    lbResult.Clear;
    FListe:=TStringList.Create;
    fcnt:=0;
    repeat
      s:=ReadNxtStr (t,';');
      FListe.Clear;
      ScanDir (SourceDirEdit.Text,s);
      fcnt:=fcnt+FListe.Count;
      with FListe do if Count>0 then for i:=0 to Count-1 do begin
        fName:=Strings[i];
        lbResult.Items.Add(_('File: ')+fName);
        InStream := TFileStream.Create(fName,fmOpenRead or fmShareDenyNone);
        ok:=true; k:=0;
        with InStream do if rgMode.ItemIndex=0 then begin // check binary
          while Position<Size do begin
            sp:=Position;
            repeat Read(w,SizeOf(word)) until (w<>0) or (Position>=Size);   // block count
            if Position<Size then begin  // read next block
              cs:=w;
              if w=1 then begin // start block
                Read(wa,SizeOf(word));   // start address
                cs:=cs+wa;
                Read(w,SizeOf(word));   // checksum
                cs:=cs+w;
                if cbShow.Checked then with lbResult do
                  ItemIndex:=Items.Add(' Start at '+IntToVal(wa,nmOctal));
                end
              else begin
                n:=65536-w;
                if n<=16 then begin  // data block
                  Read(wa,SizeOf(word));   // address
                  cs:=cs+wa;
                  Read(w,SizeOf(word));   // checksum
                  cs:=cs+w;
                  for j:=1 to n do begin
                    Read(w,SizeOf(word));   // data
                    if cbShow.Checked then begin
                      with lbResult do ItemIndex:=Items.Add(' '+IntToVal(wa,nmOctal)+
                        ' '+IntToVal(w,nmOctal)+ ' '+DisAssemble(wa,w,nmOctal,true));
                      inc(wa);
                      Application.ProcessMessages;
                      end;
                    cs:=cs+w;
                    end;
                  end
                else begin   // multiple data block
                  Read(wa,SizeOf(word));   // address
                  cs:=cs+wa;
                  Read(w,SizeOf(word));   // checksum
                  cs:=cs+w;
                  Read(w,SizeOf(word));   // data word
                  cs:=cs+w;
                  if cbShow.Checked then for j:=1 to n-1 do begin
                    m:=lbResult.Items.Add(' '+IntToVal(wa,nmOctal)+
                      ' '+IntToVal(w,nmOctal));
                    inc(wa);
                    end;
                  lbResult.ItemIndex:=m;
                  Application.ProcessMessages;
                  end;
                end;
              if cs<>0 then begin
                lbResult.Items.Add('  '+Format(_('Error in block at position: %u ($%x)'),[sp,sp]));
                Application.ProcessMessages;
                ok:=false;
                end;
              inc(k);
              end;
            end;
          if ok then lbResult.Items.Add('  '+Format(_('Checked: %u blocks: OK'),[k]))
          else lbResult.Items.Add('  '+Format(_('Checked: %u blocks: Errors found'),[k]));
          Application.ProcessMessages;
          InStream.Free;
          end
        else begin //check text
          oName:=fName+TmpExt; sd:='';
          OutStream:=TFileStream.Create(oName,fmCreate or fmShareExclusive);
          while Position<Size do begin
            Read(b,1);   // byte
            if (b<>0) then begin
              ok:=(b and $80)=0;
              b:=b and $7F;
              if (b<>$7F) then begin
                if cbShow.Checked then begin
                  if length(sd)>=40 then begin
                    lbResult.Items.Add(' '+sd);
                    sd:='';;
                    end
                  else sd:=sd+char(b);
                  end;
                OutStream.Write(b,1);
                end;
              end;
            end;
          if not ok then lbResult.Items.Add('  '+_('Parity removed'));
          lbResult.Items.Add('  '+Format(_('File stripped by %u bytes'),[Size-OutStream.Size]));
          Application.ProcessMessages;
          OutStream.Free;
          InStream.Free;
          s:=fName+BakExt;
          if FileExists(s) then DeleteFile(s);
          RenameFile(fName,s);
          RenameFile(oName,fName);
          end;
        end;
      until length(t)=0;
    FListe.Free;
    with lbResult do ItemIndex:=Items.Add(Format(_('%u Files checked'),[fcnt]));
    Screen.Cursor:=crDefault;
    end
  else lbResult.Items.Add(_('Nothing to do!'));
  end;

end.
