(* Nova-Hilfsprogramm
   ==================
   Load paper tape and save as file
   A paper tape reader must be connected to an paralle (LPT) port

    J. Rathlev, Schwentinental, (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 -  Mar. 2010
   last modified:  May 2024
   *)

unit ReadPtrMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.ExtCtrls, LptSelect;

type
  THauptForm = class(TForm)
    edFile: TLabeledEdit;
    btFile: TBitBtn;
    btStart: TBitBtn;
    btQuit: TBitBtn;
    btPort: TBitBtn;
    SaveDialog: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btQuitClick(Sender: TObject);
    procedure btPortClick(Sender: TObject);
    procedure btFileClick(Sender: TObject);
    procedure btStartClick(Sender: TObject);
  private
    { Private-Deklarationen }
    IniName               : string;
    LptNr,PortAdr,PortIrq : integer;
    Buffer                : array of byte;
    procedure IntCallBack;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.dfm}

uses TVicHW, PathUtils, WinUtils, MsgDialogs, WinShell, ShlObj, IniFiles;

(* ----------------------------------------------------------------------- *)
var
  Stopped,Fnd : boolean;
  ByteCount   : integer;

(* bidir-Mode einschalten *)
procedure InitRead (PortAdr : integer);
var
  PC : word ;
begin
  (* set ECP-Control-Reg. to bidir. Mode *)
  WritePort(PortAdr+$402,(ReadPort(PortAdr+$402) and $1F) or $20) ;
  PC:=PortAdr+2;
  { Bits in PC sind:           5   4   3   2   1   0
                              R/W IRQ DSL INI ALF STB}
  WritePort(PC,$0B xor $01); { L   L   L   L   L   H   Grundstellung }
  WritePort(PC,$0B xor $21); { H   L   L   L   L   H    auf Lesen }
  end;

(* Daten an Schnittstelle senden *)
procedure WriteData (PortAdr : integer; Value : byte);
var
  PC : word;
begin
  PC:=PortAdr+2;
  { Bits in PC sind:           5   4   3   2   1   0
                              R/W IRQ DSL INI ALF STB}
  WritePort(PC,$0B xor $00); { L   L   L   L   L   L    Grundstellung }
  WritePort(PC,$0B xor $01); { L   L   L   L   L   H    Strobe }
  WritePort(PortAdr,Value);  { Daten ausgeben }
  WritePort(PC,$0B xor $00); { L   L   L   L   L   L    Grundstellung }
  end;

(* von Schnittstelle lesen *)
function ReadData (PortAdr : integer) : byte;
var
  PC : word;
begin
  PC:=PortAdr+2;
  { Bits in PC sind:           5   4   3   2   1   0
                              R/W IRQ DSL INI ALF STB}
  ReadData:=ReadPort(PortAdr);     { Daten lesen...   }
  end;

(* Dateninput mit Handshake
   - Setze STROBE
   - Warte bis ACK *)
procedure StartRead (PortAdr : integer);
var
  PC : word;
begin
  PC:=PortAdr+2;
  { Bits in PC sind:           5   4   3   2   1   0
                              R/W IRQ DSL INI ALF STB}
  WritePort(PC,$0B xor $21); { H   L   L   L   L   H    Grundstellung }
  sleep(4);
  WritePort(PC,$0B xor $20); { H   L   L   L   L   L    Strobe }
  sleep(1);
  WritePort(PC,$0B xor $21); { H   L   L   L   L   H    Grundstellung }
  sleep(1);
  end;

(* ----------------------------------------------------------------------- *)
const
  (* INI-Sektionen *)
  CfgSekt  = 'Config';

  (* INI-Variablen *)
  iniLptNr = 'LptNr';
  iniPortAdr = 'PortAdr';
  iniPortIrq = 'PortIrq';
  iniFilename = 'Filename';

  IniExt = 'ini';

procedure THauptForm.FormCreate(Sender: TObject);
begin
  // Einstellungen laden
  IniName:=Erweiter(GetDesktopFolder(CSIDL_APPDATA),ExtractFileName(DelExt(Application.ExeName)),IniExt);
  with TMemIniFile.Create(IniName) do begin
    LptNr:=ReadInteger(CfgSekt,iniLptNr,1);
    PortAdr:=ReadInteger(CfgSekt,iniPortAdr,-1);
    PortIrq:=ReadInteger(CfgSekt,iniPortIrq,-1);
    edFile.Text:=ReadString(CfgSekt,iniFilename,'');
    Free;
    end;
  Buffer:=nil;
  Stopped:=true;
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
begin
  with TMemIniFile.Create(IniName) do begin
    WriteInteger(CfgSekt,iniLptNr,LptNr);
    WriteInteger(CfgSekt,iniPortAdr,PortAdr);
    WriteInteger(CfgSekt,iniPortIrq,PortIrq);
    WriteString(CfgSekt,iniFilename,edFile.Text);
    UpdateFile;
    Free;
    end;
  DisableInterrupt(PortIrq);
  CloseDriver;
  end;

procedure THauptForm.FormShow(Sender: TObject);
begin
  if OpenDriver(0) then begin
    ErrorDialog('Driver could not be loaded!');
    Close;
    end
  else begin
    if (LptNr<1) or (LptNr>LptPortMax) then begin
      if (PortAdr<0) or (PortIrq<0) then begin
        if (PortAdr<0) then PortAdr:=$378;
        if (PortIrq<0) then PortIrq:=7;
        if not InputLptPort (Point(Left,Top),'Port of PTR connection','',
                      LptNr,PortAdr,PortIrq) then Close;
        end
      end
    else begin
      PortAdr:=LptPort[LptNr];
      PortIrq:=LptIrq[LptNr];
      end;
   { 1284-kompatiblen Port in den "PS/2"-Mode setzen.
     Damit das ECR-Register bei Offset $402 freigeschaltet ist,
     muss im BIOS "EPP/EPC" eingestellt werden sein. }
    InitRead(PortAdr);
    EnableInterrupt (PortIrq,IntCallBack);
    end;
  end;

procedure THauptForm.btQuitClick(Sender: TObject);
begin
  Close;
  end;

procedure THauptForm.btPortClick(Sender: TObject);
begin
  if InputLptPort (CursorPos,'Port of PTR connection','',
                LptNr,PortAdr,PortIrq) then begin
    end;
  end;

procedure THauptForm.btFileClick(Sender: TObject);
begin
  with SaveDialog do begin
    InitialDir:=ExtractFilePath(edFile.Text);
    Filter:='Binary files|*.bin|all|*.*';
    Filename:='';
    Title:='Save paper tape as file';
    if Execute then edFile.Text:=Filename;
    end;
  end;

(* -------------------------------------------------------------------------
   Interrupt callback routine *)
procedure THauptForm.IntCallBack;
var
  b : byte;
begin
  b:=ReadData(PortAdr);
  if b=0 then begin
    if fnd then inc(ByteCount);
    end
  else begin
    ByteCount:=0; Fnd:=true;
    end;
  Stopped:=ByteCount>100;
  if Fnd then begin
    SetLength(Buffer,length(Buffer)+1);
    Buffer[High(Buffer)]:=b;
    end;
  if not Stopped then StartRead (PortAdr);
  end;

procedure THauptForm.btStartClick(Sender: TObject);
var
  i  : integer;
  fs : TFileStream;
begin
  if Stopped then begin
    Buffer:=nil;
    Stopped:=false; Fnd:=false;
    StartRead (PortAdr);
    repeat
      sleep(10);
      Application.ProcessMessages;
      until Stopped;
    fs:=TFileStream.Create(edFile.Text,fmCreate);
    fs.Write(Buffer[0],length(Buffer));
    fs.Free;
    end
  else Stopped:=true;
  end;

end.
