(* Nova-Simulator
   ==============
   Show breakpoints

    J. Rathlev, Schwentinental, (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   last modified:  August 2017
   *)

unit ShowBreakpoints;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.ExtCtrls, NumberUtils;

type
  TBpWin = class(TForm)
    pnRight: TPanel;
    btClear: TBitBtn;
    rbBin: TRadioButton;
    rbOctal: TRadioButton;
    rbDecimal: TRadioButton;
    rbHex: TRadioButton;
    btDel: TBitBtn;
    btClose: TBitBtn;
    lvBp: TListView;
    btAdd: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure rbBinClick(Sender: TObject);
    procedure rbOctalClick(Sender: TObject);
    procedure rbDecimalClick(Sender: TObject);
    procedure rbHexClick(Sender: TObject);
    procedure lvBpData(Sender: TObject; Item: TListItem);
    procedure lvBpDblClick(Sender: TObject);
    procedure btAddClick(Sender: TObject);
    procedure btDelClick(Sender: TObject);
    procedure btClearClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private-Deklarationen }
    FIniName,FIniSection : string;
    NumFormat : TNumMode;
  public
    { Public-Deklarationen }
    procedure LoadFromIni(IniName, Section : string);
    procedure UpdateView(ACount : integer);
  end;

var
  BpWin: TBpWin;

implementation

{$R *.dfm}

uses NovaMain, NovaUtils, WinUtils, GnuGetText, InpValue, System.IniFiles, MsgDialogs;

// Read size and position
procedure TBpWin.LoadFromIni(IniName, Section : string);
begin
  FIniName:=IniName; FIniSection:=Section;
  if (length(FIniName)>0) and (length(FIniSection)>0) then
       with TMemIniFile.Create(IniName) do begin
    Top:=ReadInteger(FIniSection,iniTop,Top);
    Left:=ReadInteger(FIniSection,iniLeft,Left);
    Height:=ReadInteger(FIniSection,iniHeight,Height);
    Width:=ReadInteger(FIniSection,iniWidth,Width);
    Free;
    end;
  end;

procedure TBpWin.lvBpData(Sender: TObject; Item: TListItem);
var
  na,ni : word;
begin
  with MainForm,Item do begin
    na:=word(Breakpoints.Objects[index]);
    Caption:=IntToVal(na,NumFormat);
    ni:=ReadMemory(na);
    SubItems.Add(IntToVal(ni,NumFormat));
    SubItems.Add(DisAssemble(na,ni,nmOctal,true));
    end;
  end;

procedure TBpWin.lvBpDblClick(Sender: TObject);
begin
  with Mainform do ShowMemory(word(Breakpoints.Objects[lvBp.Selected.Index]));
  end;

procedure TBpWin.FormCreate(Sender: TObject);
begin
  TranslateComponent(self);
  NumFormat:=nmOctal;
  end;

procedure TBpWin.FormDestroy(Sender: TObject);
begin
  if (length(FIniName)>0) and (length(FIniSection)>0) then
       with TMemIniFile.Create(FIniName) do begin
    WriteInteger(FIniSection,iniTop,Top);
    WriteInteger(FIniSection,iniLeft,Left);
    WriteInteger(FIniSection,iniHeight,Height);
    WriteInteger(FIniSection,iniWidth,Width);
    UpdateFile;
    Free;
    end;
  end;

procedure TBpWin.FormResize(Sender: TObject);
var
  w : integer;
begin
  with lvBp do begin
    w:=(Width-Columns[2].Width-21) div 2;
    Columns[0].Width:=w; Columns[1].Width:=w;
    end;
  end;

procedure TBpWin.btAddClick(Sender: TObject);
var
  na : word;
begin
  na:=0;
  if ValueDialog.Execute(BottomLeftPos(btAdd),_('Add breakpoint:'),NumFormat,na) then
    MainForm.SetBreakpoint(na);
  MainForm.InvalidateMemView;
  end;

procedure TBpWin.btClearClick(Sender: TObject);
begin
  if ConfirmDialog(_('Delete all breakpoints')+'?') then begin
    MainForm.BreakPoints.Clear;
    UpdateView(0);
    MainForm.InvalidateMemView;
    end;
  end;

procedure TBpWin.btDelClick(Sender: TObject);
var
  n : integer;
begin
  with lvBp do begin
    n:=ItemIndex;
    UpdateView(Mainform.DeleteBreakpoint(Selected.Caption));
    if n>=Items.Count then n:=Items.Count-1;
    ItemIndex:=n;
    MainForm.InvalidateMemView;
    end;
  end;

procedure TBpWin.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  MainForm.itmBreakpoints.Checked:=false;
  end;

procedure TBpWin.rbBinClick(Sender: TObject);
begin
  NumFormat:=nmBin;
  lvBp.Invalidate;
  end;

procedure TBpWin.rbDecimalClick(Sender: TObject);
begin
  NumFormat:=nmZeroDec;
  lvBp.Invalidate;
  end;

procedure TBpWin.rbHexClick(Sender: TObject);
begin
  NumFormat:=nmHex;
  lvBp.Invalidate;
  end;

procedure TBpWin.rbOctalClick(Sender: TObject);
begin
  NumFormat:=nmOctal;
  lvBp.Invalidate;
  end;

procedure TBpWin.UpdateView(ACount : integer);
begin
  with lvBp do begin
    Items.Count:=ACount;
    if Visible and (Items.Count>0) then begin
      ItemIndex:=Items.Count-1;
      Selected.MakeVisible(false);
      end;
    Invalidate;
    end;
  end;

end.
