(* Nova-Simulator
   ==============
   Show program trace

    J. Rathlev, Schwentinental, (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   last modified:  August 2017
   *)

unit ShowTrace;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ComCtrls, NumberUtils;

const
  MaxTraceItems = 10000;

type
  TTraceWin = class(TForm)
    lvTrace: TListView;
    pnBottom: TPanel;
    btClose: TBitBtn;
    btClear: TBitBtn;
    rbBin: TRadioButton;
    rbOctal: TRadioButton;
    rbDecimal: TRadioButton;
    rbHex: TRadioButton;
    procedure lvTraceData(Sender: TObject; Item: TListItem);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure btClearClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure rbBinClick(Sender: TObject);
    procedure rbOctalClick(Sender: TObject);
    procedure rbDecimalClick(Sender: TObject);
    procedure rbHexClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure lvTraceDblClick(Sender: TObject);
  private
    { Private-Deklarationen }
    FIniName,FIniSection : string;
    NumFormat : TNumMode;
  public
    { Public-Deklarationen }
    procedure LoadFromIni(IniName, Section : string);
    procedure UpdateView(ACount : integer);
  end;

var
  TraceWin: TTraceWin;

implementation

{$R *.dfm}

uses NovaMain, NovaUtils, GnuGetText, System.IniFiles;

// Read size and position
procedure TTraceWin.LoadFromIni(IniName, Section : string);
begin
  FIniName:=IniName; FIniSection:=Section;
  if (length(FIniName)>0) and (length(FIniSection)>0) then
       with TMemIniFile.Create(IniName) do begin
    Top:=ReadInteger(FIniSection,iniTop,Top);
    Left:=ReadInteger(FIniSection,iniLeft,Left);
    Height:=ReadInteger(FIniSection,iniHeight,Height);
    Width:=ReadInteger(FIniSection,iniWidth,Width);
    Free;
    end;
  end;

procedure TTraceWin.FormCreate(Sender: TObject);
begin
  TranslateComponent(self);
  NumFormat:=nmOctal;
  end;

procedure TTraceWin.FormDestroy(Sender: TObject);
begin
  if (length(FIniName)>0) and (length(FIniSection)>0) then
       with TMemIniFile.Create(FIniName) do begin
    WriteInteger(FIniSection,iniTop,Top);
    WriteInteger(FIniSection,iniLeft,Left);
    WriteInteger(FIniSection,iniHeight,Height);
    WriteInteger(FIniSection,iniWidth,Width);
    UpdateFile;
    Free;
    end;
  end;

procedure TTraceWin.FormResize(Sender: TObject);
var
  w : integer;
begin
  with lvTrace do begin
    w:=(Width-Columns[2].Width-Columns[1].Width-21) div 5;
    Columns[0].Width:=w;
    Columns[3].Width:=w; Columns[4].Width:=w;
    Columns[5].Width:=w; Columns[6].Width:=w;
    end;
  end;

procedure TTraceWin.UpdateView(ACount : integer);
begin
  with lvTrace do begin
    Items.Count:=ACount;
    if Visible and (Items.Count>0) then begin
      ItemIndex:=Items.Count-1;
      Selected.MakeVisible(false);
      end;
    Invalidate;
    end;
  end;

procedure TTraceWin.lvTraceData(Sender: TObject; Item: TListItem);
begin
  with MainForm,Item,(MainForm.Trace[index] as TTraceData) do begin
    Caption:=IntToVal(PC,NumFormat);
    SubItems.Add(DisAssemble(PC,Data,nmOctal,true));
    SubItems.Add(IntToOctal(Carry,1,0));
    SubItems.Add(IntToVal(Acs[0],NumFormat));
    SubItems.Add(IntToVal(Acs[1],NumFormat));
    SubItems.Add(IntToVal(Acs[2],NumFormat));
    SubItems.Add(IntToVal(Acs[3],NumFormat));
    end;
  end;

procedure TTraceWin.lvTraceDblClick(Sender: TObject);
var
  adr : word;
begin
   Mainform.ShowMemory(NumStrToVal(lvTrace.Selected.Caption,NumFormat));
  end;

procedure TTraceWin.rbBinClick(Sender: TObject);
begin
  NumFormat:=nmBin;
  lvTrace.Invalidate;
 end;

procedure TTraceWin.rbDecimalClick(Sender: TObject);
begin
  NumFormat:=nmZeroDec;
  lvTrace.Invalidate;
  end;

procedure TTraceWin.rbHexClick(Sender: TObject);
begin
  NumFormat:=nmHex;
  lvTrace.Invalidate;
  end;

procedure TTraceWin.rbOctalClick(Sender: TObject);
begin
  NumFormat:=nmOctal;
  lvTrace.Invalidate;
  end;

procedure TTraceWin.btCloseClick(Sender: TObject);
begin
  Close;
  end;

procedure TTraceWin.btClearClick(Sender: TObject);
begin
  MainForm.Trace.Clear;
  UpdateView(0);
  end;

procedure TTraceWin.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  MainForm.itmShowTrace.Checked:=false;
  end;

end.
