(* Nova-Simulator
   ==============
   Show watch list

    J. Rathlev, Schwentinental, (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   last modified:  August 2017
   *)

unit ShowWatch;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.ExtCtrls, NumberUtils, System.ImageList, Vcl.ImgList;

type
  TWatchWin = class(TForm)
    pnBottom: TPanel;
    btClose: TBitBtn;
    btClear: TBitBtn;
    rbBin: TRadioButton;
    rbOctal: TRadioButton;
    rbDecimal: TRadioButton;
    rbHex: TRadioButton;
    lvWatch: TListView;
    imgState: TImageList;
    imgSmall: TImageList;
    btnDel: TBitBtn;
    btnAdd: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lvWatchData(Sender: TObject; Item: TListItem);
    procedure btCloseClick(Sender: TObject);
    procedure btClearClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure rbBinClick(Sender: TObject);
    procedure rbOctalClick(Sender: TObject);
    procedure rbDecimalClick(Sender: TObject);
    procedure rbHexClick(Sender: TObject);
    procedure btnDelClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure lvWatchMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
  private
    { Private-Deklarationen }
    FIniName,FIniSection : string;
    NumFormat : TNumMode;
  public
    { Public-Deklarationen }
    procedure LoadFromIni(IniName, Section : string);
    procedure UpdateView(ACount : integer);
  end;

var
  WatchWin: TWatchWin;

implementation

{$R *.dfm}

uses NovaMain, NovaUtils, InpValue, WinUtils, GnuGetText, System.IniFiles, MsgDialogs;

// Read size and position
procedure TWatchWin.LoadFromIni(IniName, Section : string);
begin
  FIniName:=IniName; FIniSection:=Section;
  if (length(FIniName)>0) and (length(FIniSection)>0) then
       with TMemIniFile.Create(IniName) do begin
    Top:=ReadInteger(FIniSection,iniTop,Top);
    Left:=ReadInteger(FIniSection,iniLeft,Left);
    Height:=ReadInteger(FIniSection,iniHeight,Height);
    Width:=ReadInteger(FIniSection,iniWidth,Width);
    Free;
    end;
  end;

procedure TWatchWin.FormCreate(Sender: TObject);
begin
  TranslateComponent(self);
  NumFormat:=nmOctal;
  end;

procedure TWatchWin.FormDestroy(Sender: TObject);
begin
  if (length(FIniName)>0) and (length(FIniSection)>0) then
       with TMemIniFile.Create(FIniName) do begin
    WriteInteger(FIniSection,iniTop,Top);
    WriteInteger(FIniSection,iniLeft,Left);
    WriteInteger(FIniSection,iniHeight,Height);
    WriteInteger(FIniSection,iniWidth,Width);
    UpdateFile;
    Free;
    end;
  end;

procedure TWatchWin.FormResize(Sender: TObject);
var
  w : integer;
begin
  with lvWatch do begin
    w:=(Width-Columns[0].Width-Columns[3].Width-21) div 3;
    Columns[1].Width:=w; Columns[2].Width:=2*w;
    end;
  end;

procedure TWatchWin.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  MainForm.itmShowWatch.Checked:=false;
  end;

procedure TWatchWin.btCloseClick(Sender: TObject);
begin
  Close;
  end;

procedure TWatchWin.lvWatchMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  n : integer;
begin
  with lvWatch do if Items.Count>0 then begin
    if x<Columns[0].Width then begin //(htOnStateIcon in GetHitTestInfoAt(x,y)) then begin
      n:=GetItemAt(x,y).Index;
      if n>=0 then with (MainForm.Watch.Objects[n] as TWatchMem) do begin
        Alert:=not Alert;
        if Alert then Items[n].StateIndex:=1 else Items[n].StateIndex:=0;
        UpdateItems(n,n);
        end;
      end;
    end;
  end;

procedure TWatchWin.lvWatchData(Sender: TObject; Item: TListItem);
begin
  with MainForm,Item,(Watch.Objects[index] as TWatchMem) do begin
    Caption:='';
    SubItems.Add(IntToVal(Addr,NumFormat));
    SubItems.Add(IntToVal(Data,NumFormat)+
      ' ('+IntToByte(Data div 256,NumFormat)+' '+IntToByte(Data and $FF,NumFormat)+')');
    SubItems.Add(DisAssemble(Addr,Data,nmOctal,true));
    if Alert then StateIndex:=1 else StateIndex:=0;
    if Changed then ImageIndex:=1 else ImageIndex:=0;
    end;
  end;

procedure TWatchWin.btClearClick(Sender: TObject);
begin
  if (MainForm.Watch.Count>0) and ConfirmDialog(_('Delete all items from watch list')+'?') then begin
    MainForm.ClearWatches;
    UpdateView(0);
    end;
  end;

procedure TWatchWin.btnAddClick(Sender: TObject);
var
  na : word;
begin
  na:=0;
  if ValueDialog.Execute(BottomLeftPos(btnAdd),_('Add location to watch list:'),NumFormat,na) then
    MainForm.AddWatch(na);
  end;

procedure TWatchWin.btnDelClick(Sender: TObject);
var
  n : integer;
begin
  with lvWatch do if ItemIndex>=0 then begin
    n:=ItemIndex;
    UpdateView(Mainform.DeleteWatch(Selected.SubItems[0]));
    if Items.Count=0 then n:=-1
    else if n>=Items.Count then n:=Items.Count-1;
    ItemIndex:=n;
    end;
  end;

procedure TWatchWin.rbBinClick(Sender: TObject);
begin
  NumFormat:=nmBin;
  lvWatch.Invalidate;
  end;

procedure TWatchWin.rbDecimalClick(Sender: TObject);
begin
  NumFormat:=nmZeroDec;
  lvWatch.Invalidate;
  end;

procedure TWatchWin.rbHexClick(Sender: TObject);
begin
  NumFormat:=nmHex;
  lvWatch.Invalidate;
  end;

procedure TWatchWin.rbOctalClick(Sender: TObject);
begin
  NumFormat:=nmOctal;
  lvWatch.Invalidate;
  end;

procedure TWatchWin.UpdateView(ACount : integer);
begin
  with lvWatch do begin
    Items.Count:=ACount;
    if Visible and (Items.Count>0) then begin
      ItemIndex:=Items.Count-1;
      Selected.MakeVisible(false);
      end;
    Invalidate;
    end;
  end;

end.
