(* Delphi-Unit
   Auswahl eines Drucker-Ports (1 .. 4) oder individ. Angabe von Port und Irq

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Mai 2002
   last modified: Nov. 2021
   *)


unit LptSelect;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms, 
  Vcl.Controls, Vcl.StdCtrls, Vcl.Dialogs, Vcl.Buttons, Vcl.ExtCtrls, NumberEd;

const
  (* Standard-LPT *)
  LPTPortMax = 4;
  LPTPort: array [1..LPTPortMax] of word = ($378,$278,$3BC,$2BC);
  LPTIRQ : array [1..LPTPortMax] of byte = (7,5,3,9);

type
  TInputLptPortDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    LptSel: TRadioGroup;
    Panel1: TPanel;
    Descriptor1: TLabel;
    PortEdit: TRangeEdit;
    IrqEdit: TRangeEdit;
    udPort: TNumUpDown;
    Descriptor2: TLabel;
    udIrq: TNumUpDown;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure LptSelClick(Sender: TObject);
    procedure PortChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    AllowChange : boolean;
  public
    { Public declarations }
  end;

(* Zahl eingeben, Ergebnis: "true" bei "ok" *)
function InputLptPort(APos          : TPoint;
                      const Titel,ACancel : string;
                      var LptNr,PortAdr,PortIrq : integer) : boolean;

var
  InputLptPortDialog: TInputLptPortDialog;

implementation

{$R *.DFM}

uses WinUtils, MsgDialogs, GnuGetText;

{ ------------------------------------------------------------------- }
procedure TInputLptPortDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TInputLptPortDialog.LptSelClick(Sender: TObject);
begin
  with LptSel do if ItemIndex<4 then begin
    AllowChange:=false;
    PortEdit.Value:=LptPort[Itemindex+1];
    IrqEdit.Value:=LptIrq[Itemindex+1];
    AllowChange:=true;
    PortChange(Sender);
    end;
  end;

procedure TInputLptPortDialog.PortChange(Sender: TObject);
var
  i   : integer;
begin
  if AllowChange then begin
    i:=1;
    while (i<=LptPortMax) and ((PortEdit.Value<>LptPort[i]) or (IrqEdit.Value<>LptIrq[i])) do inc(i);
    with LptSel do if i<=LptPortMax then ItemIndex:=i-1 else ItemIndex:=4
    end;
  end;

procedure TInputLptPortDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  s : string;
begin
  with PortEdit do begin
    if (ModalResult=mrOK) and RangeError then begin
      if Digits>0 then s:='$'+IntToHex(MinValue,Digits)+' , $'+IntToHex(MaxValue,Digits)
      else s:=IntToStr(MinValue)+' .. '+IntToStr(MaxValue);
      with ClientOrigin do ErrorDialog (Point(X+Width,Y+Height),Format(dgettext('dialogs','Error: Invalid port address (%s)'),[s]));
      CanClose:=false;
      SetFocus;
      end
    else CanClose:=true;
    end;
  if CanClose then with IrqEdit do begin
    if (ModalResult=mrOK) and RangeError then begin
      if Digits>0 then s:='$'+IntToHex(MinValue,Digits)+' , $'+IntToHex(MaxValue,Digits)+')'
      else s:=IntToStr(MinValue)+' .. '+IntToStr(MaxValue)+')';
      with ClientOrigin do ErrorDialog (Point(X+Width,Y+Height),Format(dgettext('dialogs','Error: Invalid interrupt (%s)'),[s]));
      CanClose:=false;
      SetFocus;
      end
    else CanClose:=true;
    end;
  end;

{ ------------------------------------------------------------------- }
(* LPT-Port auswhlen *)
function InputLptPort(APos           : TPoint;
                      const Titel,ACancel : string;
                      var LptNr,PortAdr,PortIrq : integer) : boolean;
begin
  if not assigned(InputLptPortDialog) then
    InputLptPortDialog:=TInputLptPortDialog.Create(Application);
  AdjustFormPosition(Screen,InputLptPortDialog,APos);
  with InputLptPortDialog do begin
    Caption:=Titel;
    if length(ACancel)>0 then CancelBtn.Caption:=ACancel;
    with LptSel do if (LptNr<1) or (LptNr>LptPortMax) then ItemIndex:=4
    else begin
      if (LptPort[LptNr]=PortAdr) and (LptIrq[LptNr]=PortIrq) then ItemIndex:=LptNr-1
      else ItemIndex:=4;
      end;
    AllowChange:=true;
    PortEdit.Value:=PortAdr;
    IrqEdit.Value:=PortIrq;
    ActiveControl:=PortEdit;
    if ShowModal=mrOK then begin
      with LptSel do if ItemIndex=4 then LptNr:=0 else LptNr:=ItemIndex+1;
      PortAdr:=PortEdit.Value;
      PortIrq:=IrqEdit.Value;
      Result:=true;
      end
    else Result:=false;
    Release;
    end;
  InputLptPortDialog:=nil;
  end;

end.
