{========================================================================}
{=================  TVicHW32  Shareware Version 5.0     =================}
{=====          Copyright (c) 1997-2000 Victor I.Ishikeev           =====}
{==========             mailto:ivi.ufanet.ru                      =======}
{========================================================================}
{=====                 TVicHW32.DLL interface                       =====}
{========================================================================}
(* This version contains only the following parts:
   1. Port I/O
   2. Interrupt

   Changes:
      Dynamic load of DLL from main program required
      (Dynamic load gives the user the possibility to check if the DLL was found:
       functions: TVicHW32Loaded = true or LoadTVicHW32 = true)

  J. Rathlev, Apr. 2002)
  *)

unit TVicLibPort;

{==} interface {==}

uses Windows, HW_Types;

type
  TOpenTVicHW32  = function  (HW32 : THANDLE;
                       ServiceName : PChar;
                       EntryPoint : PChar) : THANDLE; stdcall;
  TCloseTVicHW32 = function  (HW32 : THANDLE) : THANDLE; stdcall;
  TGetActiveHW   = function  (HW32 : THANDLE) : BOOL; stdcall;
  TGetHardAccess = function  (HW32 : THANDLE) : BOOL; stdcall;
  TSetHardAccess = procedure (HW32 : THANDLE; bNewValue : BOOL); stdcall;
  TGetPortByte   = function  (HW32 : THANDLE; PortAddr : DWORD) : Byte; stdcall;
  TSetPortByte   = procedure (HW32 : THANDLE; PortAddr : DWORD; nNewValue : Byte); stdcall;
  TGetPortWord   = function  (HW32 : THANDLE; PortAddr : DWORD) : Word; stdcall;
  TSetPortWord   = procedure (HW32 : THANDLE; PortAddr : DWORD; nNewValue : Word); stdcall;
  TGetPortLong   = function  (HW32 : THANDLE; PortAddr : DWORD): LongInt; stdcall;
  TSetPortLong   = procedure (HW32 : THANDLE; PortAddr : DWORD; nNewValue : Longint); stdcall;
  TIsIRQMasked   = function  (HW32 : THANDLE; IRQNumber : WORD) : BOOL; stdcall;
  TUnmaskIRQ     = procedure (HW32 : THANDLE; IRQNumber : WORD; HWHandler : TOnHWInterrupt); stdcall;
  TMaskIRQ       = procedure (HW32 : THANDLE; IRQNumber : WORD); stdcall;
  THDDInfo       = procedure (HW32 : THANDLE; IdeNumber : Word; Master : Word; Info : pHDDInfo); stdcall;


{ ---------------------------------------------------------------- }
function TVicHW32Loaded : Boolean;
function UnLoadTVicHW32 : Boolean;
function LoadTVicHW32 : Boolean;

function  OpenTVicHW32(HW32 : THANDLE;
                       ServiceName : PChar;
                       EntryPoint : PChar) : THANDLE; stdcall;

function  CloseTVicHW32(HW32 : THANDLE) : THANDLE; stdcall;
function  GetActiveHW(HW32 : THANDLE) : BOOL; stdcall;
function  GetHardAccess(HW32 : THANDLE) : BOOL; stdcall;
procedure SetHardAccess(HW32 : THANDLE; bNewValue : BOOL); stdcall;
function  GetPortByte(HW32 : THANDLE; PortAddr : DWORD) : Byte; stdcall;
procedure SetPortByte(HW32 : THANDLE; PortAddr : DWORD; nNewValue : Byte); stdcall;
function  GetPortWord(HW32 : THANDLE; PortAddr : DWORD) : Word; stdcall;
procedure SetPortWord(HW32 : THANDLE; PortAddr : DWORD; nNewValue : Word); stdcall;
function  GetPortLong(HW32 : THANDLE; PortAddr : DWORD): LongInt; stdcall;
procedure SetPortLong(HW32 : THANDLE; PortAddr : DWORD; nNewValue : Longint); stdcall;

function  IsIRQMasked(HW32 : THANDLE; IRQNumber : WORD) : BOOL; stdcall;
procedure UnmaskIRQ(HW32 : THANDLE; IRQNumber : WORD; HWHandler : TOnHWInterrupt); stdcall;
procedure MaskIRQ(HW32 : THANDLE; IRQNumber : WORD); stdcall;

procedure GetHDDInfo(HW32 : THANDLE; IdeNumber : Word; Master    : Word;
                     Info : pHDDInfo); stdcall;

{ ---------------------------------------------------------------- }
{==} implementation {==}

var
  _OpenTVicHW32  : TOpenTVicHW32;
  _CloseTVicHW32 : TCloseTVicHW32;
  _GetActiveHW   : TGetActiveHW;
  _GetHardAccess : TGetHardAccess;
  _SetHardAccess : TSetHardAccess;
  _GetPortByte   : TGetPortByte;
  _SetPortByte   : TSetPortByte;
  _GetPortWord   : TGetPortWord;
  _SetPortWord   : TSetPortWord;
  _GetPortLong   : TGetPortLong;
  _SetPortLong   : TSetPortLong;
  _IsIRQMasked   : TIsIRQMasked;
  _UnmaskIRQ     : TUnmaskIRQ;
  _MaskIRQ       : TMaskIRQ;
  _GetHDDInfo    : THDDInfo;

function  OpenTVicHW32(HW32 : THANDLE;
                       ServiceName : PChar;
                       EntryPoint : PChar) : THANDLE; stdcall;
begin
  if assigned(_OpenTVicHW32) then Result:=_OpenTVicHW32 (HW32,ServiceName,Entrypoint)
  else Result:=0;
  end;

function  CloseTVicHW32(HW32 : THANDLE) : THANDLE; stdcall;
begin
  if assigned(_CloseTVicHW32) then Result:=_CloseTVicHW32(HW32)
  else Result:=0;
  end;

function  GetActiveHW(HW32 : THANDLE) : BOOL; stdcall;
begin
  if assigned(_GetActiveHW) then Result:=_GetActiveHW(HW32)
  else Result:=false;
  end;

function  GetHardAccess(HW32 : THANDLE) : BOOL; stdcall;
begin
  if assigned(_GetHardAccess) then Result:=_GetHardAccess(HW32)
  else Result:=false;
  end;

procedure SetHardAccess(HW32 : THANDLE; bNewValue : BOOL); stdcall;
begin
  if assigned(_SetHardAccess) then _SetHardAccess(HW32,bNewValue);
  end;

function  GetPortByte(HW32 : THANDLE; PortAddr : DWORD) : Byte; stdcall;
begin
  if assigned(_GetPortByte) then Result:=_GetPortByte(HW32,PortAddr)
  else Result:=0;
  end;

procedure SetPortByte(HW32 : THANDLE; PortAddr : DWORD; nNewValue : Byte); stdcall;
begin
  if assigned(_SetPortByte) then _SetPortByte(HW32,PortAddr,nNewValue);
  end;

function  GetPortWord(HW32 : THANDLE; PortAddr : DWORD) : Word; stdcall;
begin
  if assigned(_GetPortWord) then Result:=_GetPortWord(HW32,PortAddr)
  else Result:=0;
  end;

procedure SetPortWord(HW32 : THANDLE; PortAddr : DWORD; nNewValue : Word); stdcall;
begin
  if assigned(_SetPortWord) then _SetPortWord(HW32,PortAddr,nNewValue);
  end;

function  GetPortLong(HW32 : THANDLE; PortAddr : DWORD): LongInt; stdcall;
begin
  if assigned(_GetPortLong) then Result:=_GetPortLong(HW32,PortAddr)
  else Result:=0;
  end;

procedure SetPortLong(HW32 : THANDLE; PortAddr : DWORD; nNewValue : Longint); stdcall;
begin
  if assigned(_SetPortLong) then _SetPortLong(HW32,PortAddr,nNewValue);
  end;


function  IsIRQMasked(HW32 : THANDLE; IRQNumber : WORD) : BOOL; stdcall;
begin
  if assigned(_IsIRQMasked) then Result:=_IsIRQMasked(HW32,IRQNumber)
  else Result:=true;
  end;

procedure UnmaskIRQ(HW32 : THANDLE; IRQNumber : WORD; HWHandler : TOnHWInterrupt); stdcall;
begin
  if assigned(_UnmaskIRQ) then _UnmaskIRQ(HW32,IRQNumber,HWHandler);
  end;

procedure MaskIRQ(HW32 : THANDLE; IRQNumber : WORD); stdcall;
begin
  if assigned(_MaskIRQ) then _MaskIRQ(HW32,IRQNumber);
  end;

procedure GetHDDInfo(HW32 : THANDLE; IdeNumber : Word; Master : Word;
                     Info : pHDDInfo); stdcall;
begin
  if assigned(_GetHDDInfo) then _GetHDDInfo(HW32,IdeNumber,Master,Info);
  end;

{ ---------------------------------------------------------------- }
const
  TVicHW32DLL = 'TVicHW32.DLL';

{ ---------------------------------------------------------------- }
(* dynamic load of DLL added (J. Rathlev, Apr. 2002) *)
var
  TVicHW32Lib : THandle = 0;

function TVicHW32Loaded : Boolean;
begin
  Result:= TVicHW32Lib <> 0;
end;

function LoadTVicHW32 : Boolean;
begin
  Result:= TVicHW32Loaded;
  if (not Result) then begin
    TVicHW32Lib:= LoadLibrary(TVicHW32DLL);
    Result:=TVicHW32Loaded;
    if Result then begin
      @_OpenTVicHW32:=GetProcAddress(TVicHW32Lib,'OpenTVicHW32');
      @_CloseTVicHW32:=GetProcAddress(TVicHW32Lib,'CloseTVicHW32');
      @_GetActiveHW:=GetProcAddress(TVicHW32Lib,'GetActiveHW');

      @_GetHardAccess:=GetProcAddress(TVicHW32Lib,'GetHardAccess');
      @_SetHardAccess:=GetProcAddress(TVicHW32Lib,'SetHardAccess');

      @_GetPortByte:=GetProcAddress(TVicHW32Lib,'GetPortByte');
      @_SetPortByte:=GetProcAddress(TVicHW32Lib,'SetPortByte');
      @_GetPortWord:=GetProcAddress(TVicHW32Lib,'GetPortWord');
      @_SetPortWord:=GetProcAddress(TVicHW32Lib,'SetPortWord');
      @_GetPortLong:=GetProcAddress(TVicHW32Lib,'GetPortLong');
      @_SetPortLong:=GetProcAddress(TVicHW32Lib,'SetPortLong');

      @_IsIRQMasked:=GetProcAddress(TVicHW32Lib,'IsIRQMasked');
      @_UnmaskIRQ:=GetProcAddress(TVicHW32Lib,'UnmaskIRQ');
      @_MaskIRQ:=GetProcAddress(TVicHW32Lib,'MaskIRQ');
      @_GetHDDInfo:=GetProcAddress(TVicHW32Lib,'GetHDDInfo');
      end;
    end;
  end;

function UnLoadTVicHW32 : Boolean;
begin
  Result:= True;
  if TVicHW32Loaded then begin
    Result:= FreeLibrary(TVicHW32Lib);
    TVicHW32Lib:=0;
    end;
  end;

initialization
finalization
  UnLoadTVicHW32;
end.
